//+------------------------------------------------------------------+
//|                                           Henderson's filter.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"

#property indicator_chart_window
#property indicator_buffers 3
#property indicator_color1  PaleVioletRed
#property indicator_color2  DeepSkyBlue
#property indicator_color3  DeepSkyBlue
#property indicator_width1  3
#property indicator_width2  3
#property indicator_width3  3
#property strict

//
//
//
//
//

extern ENUM_TIMEFRAMES     TimeFrame   = PERIOD_CURRENT; // Time frame to use
extern ENUM_APPLIED_PRICE  Price       = PRICE_CLOSE;    // Price to use
extern int                 FilterSize  = 34;   //13      // Filter period
extern bool                Causal      = false;          // Causal (non-repainting) calculation?
extern bool                Interpolate = true;           // Interpolate in multi time frame mode?

//
//
//
//
//

double buffer1[];
double buffer2[];
double buffer3[];
double trend[];
double weights[];
int    half;
string indicatorFileName;
bool   returnBars;


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//

int init()
{
   IndicatorBuffers(4);
   IndicatorDigits(6);
      SetIndexBuffer(0,buffer1);
      SetIndexBuffer(1,buffer2); 
      SetIndexBuffer(2,buffer3); 
      SetIndexBuffer(3,trend);
      
      FilterSize = MathMax(FilterSize,3);
            if (MathMod(FilterSize,2)==0) FilterSize++;
      half = (int)MathFloor(FilterSize/2.0); 

      //
      //
      //
      //
      //

         indicatorFileName = WindowExpertName();
         returnBars        = (TimeFrame==-99);     if (returnBars) return(0);
         TimeFrame         = MathMax(TimeFrame,_Period);

               ArrayResize(weights,FilterSize);
               
               //
               //
               //
               //
               //
               
               double m = half;
               double divisor = 8.0*(m+2.0)*((m+2.0)*(m+2.0)-1.0)*(4.0*(m+2.0)*(m+2)-1.0)*(4.0*(m+2.0)*(m+2.0)-9.0)*(4.0*(m+2.0)*(m+2.0)-25.0);  
               for (int j=-half; j<=half; j++)
                  weights[j+half] = 315.0*((m+1.0)*(m+1.0)-(j*j))*((m+2.0)*(m+2.0)-(j*j))*((m+3.0)*(m+3.0)-(j*j))*(3.0*(m+2.0)*(m+2.0)-11.0*(j*j)-16.0)/divisor;
       
   return(0);
}
int deinit() { return(0); }



//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int limit,counted_bars=IndicatorCounted();
   int size = ArraySize(weights);

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(MathMax(Bars-counted_bars,half),Bars-1);
         if (returnBars) { buffer1[0] = limit+1; return(0); }

   //
   //
   //
   //
   //

   if (TimeFrame == Period())
   {
      if (trend[limit]==1) CleanPoint(limit,buffer2,buffer3);
      
      //
      //
      //
      //
      //
      
      for(int i=limit; i>=0; i--)
      {
         double mod = 1.0;
         if (!Causal && i<half)
         {
            double weightTotal = 0; for (int j=half-i; j<size; j++) weightTotal += weights[j]; if (weightTotal!=0) mod = 1.0/weightTotal;
         }
         
         //
         //
         //
         // 
         //

         double sum=0;
         if (!Causal)
         {         
            sum = mod*weights[half]*iMA(NULL,0,1,0,MODE_SMA,Price,i);
            for(int j=1; j<=half; j++)
            {
               if (j<=i) sum += mod*weights[half+j]*iMA(NULL,0,1,0,MODE_SMA,Price,i-j);
                         sum += mod*weights[half-j]*iMA(NULL,0,1,0,MODE_SMA,Price,i+j);
            }
         }
         else
         {
            sum = mod*weights[half]*iMA(NULL,0,1,0,MODE_SMA,Price,i+half);
            for(int j=1; j<=half; j++)
            {
               sum += mod*weights[half+j]*iMA(NULL,0,1,0,MODE_SMA,Price,i+half-j);
               sum += mod*weights[half-j]*iMA(NULL,0,1,0,MODE_SMA,Price,i+half+j);
            }
         }            
         buffer1[i] = sum;
         buffer2[i] = EMPTY_VALUE;
         buffer3[i] = EMPTY_VALUE;
         if (i<Bars-1)
         {
            if (buffer1[i]>buffer1[i+1]) trend[i] =  1;
            if (buffer1[i]<buffer1[i+1]) trend[i] = -1;
            if (trend[i] == 1) PlotPoint(i,buffer2,buffer3,buffer1);
         }            
      }            
      return(0);
   }
   
   //
   //
   //
   //
   //
   
   limit = (int)MathMax(limit,MathMin(Bars-1,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/Period()));
   if (trend[limit]==1) CleanPoint(limit,buffer2,buffer3);
   for(int i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,TimeFrame,Time[i]);
         trend[i]   = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,Price,FilterSize,Causal,3,y);
         buffer1[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,Price,FilterSize,Causal,0,y);
         buffer2[i] = EMPTY_VALUE;
         buffer3[i] = EMPTY_VALUE;
            
         //
         //
         //
         //
         //
      
         if (!Interpolate || (i>0 &&y==iBarShift(NULL,TimeFrame,Time[i-1]))) continue;

         //
         //
         //
         //
         //

         int n,k; datetime time = iTime(NULL,TimeFrame,y);
            for(n = 1; i+n<Bars && Time[i+n] >= time; n++) continue;	
            for(k = 1; i+k<Bars && i+n<Bars && k<n; k++)
               buffer1[i+k] = buffer1[i] + (buffer1[i+n]-buffer1[i])*k/n;
   }
   for(int i=limit; i>=0; i--) if (trend[i]==1) PlotPoint(i,buffer2,buffer3,buffer1);
   
   //
   //
   //
   //
   //
   
   return(0);   
}


//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if (i>=Bars-3) return;
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (i>=Bars-2) return;
   if (first[i+1] == EMPTY_VALUE)
      if (first[i+2] == EMPTY_VALUE) 
            { first[i]  = from[i];  first[i+1]  = from[i+1]; second[i] = EMPTY_VALUE; }
      else  { second[i] =  from[i]; second[i+1] = from[i+1]; first[i]  = EMPTY_VALUE; }
   else     { first[i]  = from[i];                           second[i] = EMPTY_VALUE; }
}